@@ -11,7 +11,7 @@ use warnings;
 # Not really a Notifier but we like the ->maybe_invoke_event style
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.18_001';
+our $VERSION = '0.18_002';
 $VERSION = eval $VERSION;
 
 use Carp;
@@ -121,6 +121,10 @@ Invoked on receipt of a typing notification change to give the full set of
 currently-typing members. This is invoked after the individual C<on_typing>
 events.
 
+=head2 on_read_receipt $member, $event_id, $content
+
+Invoked on receipt of a C<m.read> type of receipt message.
+
 =cut
 
 sub _init
@@ -146,7 +150,7 @@ sub configure
 
    foreach (qw( on_message on_back_message on_membership on_back_membership
          on_presence on_synced_state on_state_changed on_back_state_changed
-         on_typing on_members_typing )) {
+         on_typing on_members_typing on_read_receipt )) {
       $self->{$_} = delete $params{$_} if exists $params{$_};
    }
 
@@ -1077,6 +1081,24 @@ sub typing_stop
    );
 }
 
+=head2 send_read_receipt
+
+   $room->send_read_receipt( event_id => $event_id, ... )->get
+
+Sends a C<m.read> receipt to the given room for the given event ID.
+
+=cut
+
+sub send_read_receipt
+{
+   my $self = shift;
+   my %args = @_;
+
+   my $event_id = $args{event_id} or croak "Require event_id";
+
+   $self->_do_POST_json( "/receipt/m.read/$event_id", {} );
+}
+
 sub _handle_roomevent_create_forward
 {
    my $self = shift;
@@ -1157,6 +1179,25 @@ sub _handle_event_m_typing_ephemeral
    $self->maybe_invoke_event( on_members_typing => grep { defined } @members );
 }
 
+sub _handle_event_m_receipt_ephemeral
+{
+   my $self = shift;
+   my ( $event ) = @_;
+
+   my $content = $event->{content};
+   foreach my $event_id ( keys %$content ) {
+      my $receipt = $content->{$event_id};
+      my $read_receipt = $receipt->{"m.read"} or next;
+
+      foreach my $user_id ( keys %$read_receipt ) {
+         my $content = $read_receipt->{$user_id};
+         my $member = $self->member( $user_id ) or next;
+
+         $self->maybe_invoke_event( on_read_receipt => $member, $event_id, $content );
+      }
+   }
+}
+
 =head1 MEMBERSHIP STRUCTURES
 
 Parameters documented as C<$member> receive a membership struct, which
